@@ -18,8 +18,6 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include "config.h"
-
 #include "kstatusnotifieritem.h"
 #include "kstatusnotifieritemprivate_p.h"
 #include "kstatusnotifieritemdbus_p.h"
@@ -47,15 +45,12 @@
 
 #include <netinet/in.h>
 
-#ifdef HAVE_DBUSMENUQT
 #include <dbusmenuexporter.h>
-#endif
 
 #include "statusnotifieritemadaptor.h"
 
 static const QString s_statusNotifierWatcherServiceName("org.kde.StatusNotifierWatcher");
 
-#ifdef HAVE_DBUSMENUQT
 /**
  * Specialization to provide access to KDE icon names
  */
@@ -70,7 +65,7 @@ protected:
     virtual QString iconNameForAction(QAction *action)
     {
         KIcon icon(action->icon());
-#if QT_VERSION >= 0x040700
+#if QT_VERSION >= 0x040701
         // QIcon::name() is in the 4.7 git branch, but it is not in 4.7 TP.
         // If you get a build error here, you need to update your pre-release
         // of Qt 4.7.
@@ -102,7 +97,6 @@ protected:
 #endif
     }
 };
-#endif // HAVE_DBUSMENUQT
 
 KStatusNotifierItem::KStatusNotifierItem(QObject *parent)
       : QObject(parent),
@@ -124,7 +118,9 @@ KStatusNotifierItem::~KStatusNotifierItem()
     delete d->statusNotifierWatcher;
     delete d->notificationsClient;
     delete d->systemTrayIcon;
-    delete d->menu;
+    if (!qApp->closingDown()) {
+        delete d->menu;
+    }
     delete d;
     KGlobal::deref();
 }
@@ -157,6 +153,10 @@ void KStatusNotifierItem::setTitle(const QString &title)
 
 void KStatusNotifierItem::setStatus(const ItemStatus status)
 {
+    if (d->status == status) {
+        return;
+    }
+
     d->status = status;
     emit d->statusNotifierItemDBus->NewStatus(metaObject()->enumerator(metaObject()->indexOfEnumerator("ItemStatus")).valueToKey(d->status));
 
@@ -171,6 +171,10 @@ void KStatusNotifierItem::setStatus(const ItemStatus status)
 
 void KStatusNotifierItem::setIconByName(const QString &name)
 {
+    if (d->iconName == name) {
+        return;
+    }
+
     d->serializedIcon = KDbusImageVector();
     d->iconName = name;
     emit d->statusNotifierItemDBus->NewIcon();
@@ -186,6 +190,10 @@ QString KStatusNotifierItem::iconName() const
 
 void KStatusNotifierItem::setIconByPixmap(const QIcon &icon)
 {
+    if (d->iconName.isEmpty() && d->icon.cacheKey() == icon.cacheKey()) {
+        return;
+    }
+
     d->iconName.clear();
     d->serializedIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewIcon();
@@ -203,6 +211,10 @@ QIcon KStatusNotifierItem::iconPixmap() const
 
 void KStatusNotifierItem::setOverlayIconByName(const QString &name)
 {
+    if (d->overlayIconName == name) {
+        return;
+    }
+
     d->overlayIconName = name;
     emit d->statusNotifierItemDBus->NewOverlayIcon();
     if (d->systemTrayIcon) {
@@ -224,6 +236,11 @@ QString KStatusNotifierItem::overlayIconName() const
 
 void KStatusNotifierItem::setOverlayIconByPixmap(const QIcon &icon)
 {
+    if (d->overlayIconName.isEmpty() && d->overlayIcon.cacheKey() == icon.cacheKey()) {
+        return;
+    }
+
+    d->overlayIconName.clear();
     d->serializedOverlayIcon = d->iconToVector(icon);
     emit d->statusNotifierItemDBus->NewOverlayIcon();
 
@@ -248,6 +265,10 @@ QIcon KStatusNotifierItem::overlayIconPixmap() const
 
 void KStatusNotifierItem::setAttentionIconByName(const QString &name)
 {
+    if (d->attentionIconName == name) {
+        return;
+    }
+
     d->serializedAttentionIcon = KDbusImageVector();
     d->attentionIconName = name;
     emit d->statusNotifierItemDBus->NewAttentionIcon();
@@ -260,6 +281,10 @@ QString KStatusNotifierItem::attentionIconName() const
 
 void KStatusNotifierItem::setAttentionIconByPixmap(const QIcon &icon)
 {
+    if (d->attentionIconName.isEmpty() && d->attentionIcon.cacheKey() == icon.cacheKey()) {
+        return;
+    }
+
     d->attentionIconName.clear();
     d->serializedAttentionIcon = d->iconToVector(icon);
     d->attentionIcon = icon;
@@ -299,22 +324,51 @@ QString KStatusNotifierItem::attentionMovieName() const
 
 void KStatusNotifierItem::setToolTip(const QString &iconName, const QString &title, const QString &subTitle)
 {
-    setToolTipIconByName(iconName);
-    setToolTipTitle(title);
-    setToolTipSubTitle(subTitle);
+    if (d->toolTipIconName == iconName &&
+        d->toolTipTitle == title &&
+        d->toolTipSubTitle == subTitle) {
+        return;
+    }
+
+    d->serializedToolTipIcon = KDbusImageVector();
+    d->toolTipIconName = iconName;
+
+    d->toolTipTitle = title;
+    if (d->systemTrayIcon) {
+        d->systemTrayIcon->setToolTip(title);
+    }
+
+    d->toolTipSubTitle = subTitle;
     emit d->statusNotifierItemDBus->NewToolTip();
 }
 
 void KStatusNotifierItem::setToolTip(const QIcon &icon, const QString &title, const QString &subTitle)
 {
-    setToolTipIconByPixmap(icon);
-    setToolTipTitle(title);
-    setToolTipSubTitle(subTitle);
+    if (d->toolTipIconName.isEmpty() && d->toolTipIcon.cacheKey() == icon.cacheKey() &&
+        d->toolTipTitle == title &&
+        d->toolTipSubTitle == subTitle) {
+        return;
+    }
+
+    d->toolTipIconName.clear();
+    d->serializedToolTipIcon = d->iconToVector(icon);
+    d->toolTipIcon = icon;
+
+    d->toolTipTitle = title;
+    if (d->systemTrayIcon) {
+        d->systemTrayIcon->setToolTip(title);
+    }
+
+    d->toolTipSubTitle = subTitle;
     emit d->statusNotifierItemDBus->NewToolTip();
 }
 
 void KStatusNotifierItem::setToolTipIconByName(const QString &name)
 {
+    if (d->toolTipIconName == name) {
+        return;
+    }
+
     d->serializedToolTipIcon = KDbusImageVector();
     d->toolTipIconName = name;
     emit d->statusNotifierItemDBus->NewToolTip();
@@ -327,6 +381,10 @@ QString KStatusNotifierItem::toolTipIconName() const
 
 void KStatusNotifierItem::setToolTipIconByPixmap(const QIcon &icon)
 {
+    if (d->toolTipIconName.isEmpty() && d->toolTipIcon.cacheKey() == icon.cacheKey()) {
+        return;
+    }
+
     d->toolTipIconName.clear();
     d->serializedToolTipIcon = d->iconToVector(icon);
     d->toolTipIcon = icon;
@@ -340,6 +398,10 @@ QIcon KStatusNotifierItem::toolTipIconPixmap() const
 
 void KStatusNotifierItem::setToolTipTitle(const QString &title)
 {
+    if (d->toolTipTitle == title) {
+        return;
+    }
+
     d->toolTipTitle = title;
     emit d->statusNotifierItemDBus->NewToolTip();
     if (d->systemTrayIcon) {
@@ -354,6 +416,10 @@ QString KStatusNotifierItem::toolTipTitle() const
 
 void KStatusNotifierItem::setToolTipSubTitle(const QString &subTitle)
 {
+    if (d->toolTipSubTitle == subTitle) {
+        return;
+    }
+
     d->toolTipSubTitle = subTitle;
     emit d->statusNotifierItemDBus->NewToolTip();
 }
@@ -363,7 +429,6 @@ QString KStatusNotifierItem::toolTipSubTitle() const
     return d->toolTipSubTitle;
 }
 
-
 void KStatusNotifierItem::setContextMenu(KMenu *menu)
 {
     if (d->menu && d->menu != menu) {
@@ -379,21 +444,17 @@ void KStatusNotifierItem::setContextMenu(KMenu *menu)
     if (d->systemTrayIcon) {
         d->systemTrayIcon->setContextMenu(menu);
     } else if (d->menu != menu) {
-#ifdef HAVE_DBUSMENUQT
         if (getenv("KSNI_NO_DBUSMENU")) {
-#endif // HAVE_DBUSMENUQT
             // This is a hack to make it possible to disable DBusMenu in an
             // application. The string "/NO_DBUSMENU" must be the same as in
             // DBusSystemTrayWidget::findDBusMenuInterface() in the Plasma
             // systemtray applet.
             d->menuObjectPath = "/NO_DBUSMENU";
             menu->installEventFilter(this);
-#ifdef HAVE_DBUSMENUQT
         } else {
             d->menuObjectPath = "/MenuBar";
             new KDBusMenuExporter(d->menuObjectPath, menu, d->statusNotifierItemDBus->dbusConnection());
         }
-#endif // HAVE_DBUSMENUQT
 
         connect(menu, SIGNAL(aboutToShow()), this, SLOT(contextMenuAboutToShow()));
     }
@@ -711,7 +772,9 @@ void KStatusNotifierItemPrivate::registerToDaemon()
     if (!statusNotifierWatcher) {
         statusNotifierWatcher = new org::kde::StatusNotifierWatcher(s_statusNotifierWatcherServiceName, "/StatusNotifierWatcher",
                                                                     QDBusConnection::sessionBus());
-        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierItemUnregistered(const QString&)),
+        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierHostRegistered()),
+                         q, SLOT(checkForRegisteredHosts()));
+        QObject::connect(statusNotifierWatcher, SIGNAL(StatusNotifierHostUnregistered()),
                          q, SLOT(checkForRegisteredHosts()));
     }
 
@@ -954,8 +1017,15 @@ KDbusImageVector KStatusNotifierItemPrivate::iconToVector(const QIcon &icon)
 
     QPixmap iconPixmap;
 
+    //availableSizes() won't work on KIcon
+    QList<QSize> allSizes;
+    allSizes << QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall)
+             << QSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium)
+             << QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium)
+             << QSize(KIconLoader::SizeLarge, KIconLoader::SizeLarge);
+
     //if an icon exactly that size wasn't found don't add it to the vector
-    foreach (const QSize &size, icon.availableSizes()) {
+    foreach (const QSize &size, allSizes) {
         //hopefully huge and enormous not necessary right now, since it's quite costly
         if (size.width() <= KIconLoader::SizeLarge) {
             iconPixmap = icon.pixmap(size);
