@@ -70,8 +70,8 @@ class KTextEdit::Private
         lastReplacedPosition(-1)
     {
         //Check the default sonnet settings to see if spellchecking should be enabled.
-        sonnetKConfig = new KConfig("sonnetrc");
-        KConfigGroup group(sonnetKConfig, "Spelling");
+        KConfig sonnetKConfig("sonnetrc");
+        KConfigGroup group(&sonnetKConfig, "Spelling");
         checkSpellingEnabled = group.readEntry("checkerEnabledByDefault", false);
 
         // i18n: Placeholder text in text edit widgets is the text appearing
@@ -90,7 +90,6 @@ class KTextEdit::Private
       delete find;
       delete replace;
       delete repDlg;
-      delete sonnetKConfig;
     }
 
     /**
