@@ -224,13 +224,13 @@ void XMLHttpRequest::putValueProperty(ExecState *exec, int token, JSValue *value
 // Token according to RFC 2616
 static bool isValidFieldName(const QString& name)
 {
-    int l = name.length();
+    const int l = name.length();
     if (l == 0)
         return false;
-    
+
     const QChar* c = name.constData();
     for (int i = 0; i < l; ++i, ++c) {
-        int u = c->unicode();
+        ushort u = c->unicode();
         if (u < 32 || u > 126)
             return false;
         switch (u) {
@@ -239,7 +239,7 @@ static bool isValidFieldName(const QString& name)
         case '\\': case '"': case '/':
         case '[': case ']': case '?': case '=':
         case '{': case '}': case '\t': case ' ':
-	    return false;
+            return false;
         default:
             break;
         }
@@ -249,13 +249,13 @@ static bool isValidFieldName(const QString& name)
 
 static bool isValidFieldValue(const QString& name)
 {
-    int l = name.length();
+    const int l = name.length();
     if (l == 0)
         return true;
 
     const QChar* c = name.constData();
     for (int i = 0; i < l; ++i, ++c) {
-        int u = c->unicode();
+        ushort u = c->unicode();
         if ( u == '\n' || u == '\r' )
            return false;
     }
@@ -485,26 +485,31 @@ void XMLHttpRequest::send(const QString& _body, int& ec)
 
   // We need to use a POST-like setup even for non-post whenever we
   // have a payload.
-  if (m_method == QLatin1String("POST") || !_body.isEmpty()) {
-
+  const bool havePayload = !_body.isEmpty();
+  if (m_method == QLatin1String("POST") || havePayload) {
     // FIXME: determine post encoding correctly by looking in headers
     // for charset.
     QByteArray buf = _body.toUtf8();
-
     job = KIO::http_post( url, buf, KIO::HideProgressInfo );
-    if(contentType.isNull())
-      job->addMetaData( "content-type", "Content-type: text/plain" );
-    else
-      job->addMetaData( "content-type", contentType );
-
-  }
-  else {
+  } else if (m_method == QLatin1String("HEAD")) {
+    job = KIO::mimetype(url, KIO::HideProgressInfo);
+  } else if (m_method == QLatin1String("PUT")) {
+    job = KIO::put(url, -1, KIO::HideProgressInfo);
+  } else {
     job = KIO::get( url, KIO::NoReload, KIO::HideProgressInfo );
   }
 
   // Regardless of job type, make sure the method is set
   job->addMetaData("CustomHTTPMethod", m_method);
 
+  // Set headers
+
+  if (!contentType.isNull()) {
+    job->addMetaData("content-type", contentType);
+  } else if (havePayload) {
+    job->addMetaData("content-type", "Content-type: text/plain");
+  }
+
   if (!m_requestHeaders.isEmpty()) {
     QString rh;
     HTTPHeaderMap::ConstIterator begin = m_requestHeaders.constBegin();
@@ -607,10 +612,9 @@ void XMLHttpRequest::setRequestHeader(const QString& _name, const QString& _valu
       ec = DOMException::SYNTAX_ERR;
       return;
   }
+
   QString name = _name.toLower();
   QString value = _value.trimmed();
-  if (value.isEmpty())
-      return;
 
   // Content-type needs to be set separately from the other headers
   if(name == "content-type") {
@@ -643,7 +647,11 @@ void XMLHttpRequest::setRequestHeader(const QString& _name, const QString& _valu
       return;
   }
 
-  m_requestHeaders[_name] = value;
+  if (m_requestHeaders.contains(_name)) {
+    m_requestHeaders[_name] += (QLatin1String(", ") + value);
+  } else {
+    m_requestHeaders[_name] = value;
+  }
 }
 
 JSValue *XMLHttpRequest::getAllResponseHeaders(int& ec) const
@@ -967,14 +975,10 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
       {
       if (args.size() < 2)
           return throwError(exec, SyntaxError, "Not enough arguments");
-      JSValue* keyArgument = args[0];
-      JSValue* valArgument = args[1];
-      QString key, val;
-      if (!keyArgument->isUndefined() && !keyArgument->isNull())
-          key = keyArgument->toString(exec).qstring();
-      if (!valArgument->isUndefined() && !valArgument->isNull())
-          val = valArgument->toString(exec).qstring();
-      request->setRequestHeader(key, val, ec);
+
+      QString headerFieldName = args[0]->toString(exec).qstring();
+      QString headerFieldValue = args[1]->toString(exec).qstring();
+      request->setRequestHeader(headerFieldName, headerFieldValue, ec);
       setDOMException(exec, ec);
       return jsUndefined();
       }
