@@ -250,10 +250,13 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
         }
         case PutOperation: {
             //kDebug( 7044 ) << "PutOperation:" << reqUrl;
-            if (outgoingData)
-                kioJob = KIO::storedPut(outgoingData->readAll(), reqUrl, -1, KIO::HideProgressInfo);
-            else
+            const qint64 size = sizeFromRequest(req);
+            if (size > 0) {
+                kioJob = KIO::http_post(reqUrl, outgoingData, size, KIO::HideProgressInfo);
+                metaData.insert(QL1S("CustomHTTPMethod"), QL1S("PUT"));
+            } else {
                 kioJob = KIO::put(reqUrl, -1, KIO::HideProgressInfo);
+            }
             break;
         }
         case PostOperation: {
@@ -272,7 +275,13 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
         }
         case DeleteOperation: {
             //kDebug(7044) << "DeleteOperation:" << reqUrl;
-            kioJob = KIO::http_delete(reqUrl, KIO::HideProgressInfo);
+            const qint64 size = sizeFromRequest(req);
+            if (size > 0) {
+                kioJob = KIO::http_post(reqUrl, outgoingData, size, KIO::HideProgressInfo);
+                metaData.insert(QL1S("CustomHTTPMethod"), QL1S("DELETE"));
+            } else {
+                kioJob = KIO::http_delete(reqUrl, KIO::HideProgressInfo);
+            }
             break;
         }
         case CustomOperation: {
@@ -283,8 +292,9 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
                 return new KDEPrivate::AccessManagerReply(op, req, QNetworkReply::ProtocolUnknownError, i18n("Unknown HTTP verb."), this);
             }
 
-            if (outgoingData)
-                kioJob = KIO::http_post(reqUrl, outgoingData, sizeFromRequest(req), KIO::HideProgressInfo);
+            const qint64 size = sizeFromRequest(req);
+            if (size > 0)
+                kioJob = KIO::http_post(reqUrl, outgoingData, size, KIO::HideProgressInfo);
             else
                 kioJob = KIO::get(reqUrl, KIO::NoReload, KIO::HideProgressInfo);
 
@@ -332,6 +342,7 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
             kDebug(7044) << "Synchronous XHR:" << reply << reqUrl;
         } else {
             kWarning(7044) << "Failed to create a synchronous XHR for" << reqUrl;
+            kWarning(7044) << "REASON:" << kioJob->errorString();
             reply = new KDEPrivate::AccessManagerReply(op, req, QNetworkReply::UnknownNetworkError, kioJob->errorText(), this);
         }
     } else {
