@@ -1046,6 +1046,12 @@ void TransferJob::slotFinished()
         if (queryMetaData("permanent-redirect")=="true")
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
 
+        if (queryMetaData(QLatin1String("redirect-to-get")) == QLatin1String("true")) {
+            d->m_command = CMD_GET;
+            d->m_outgoingMetaData.remove(QLatin1String("CustomHTTPMethod"));
+            d->m_outgoingMetaData.remove(QLatin1String("content-type"));
+        }
+
         if (d->m_redirectionHandlingEnabled) {
             // Honour the redirection
             // We take the approach of "redirecting this same job"
@@ -1061,7 +1067,9 @@ void TransferJob::slotFinished()
             KUrl dummyUrl;
             QDataStream istream( d->m_packedArgs );
             switch( d->m_command ) {
-                case CMD_GET: {
+                case CMD_GET:
+                case CMD_STAT:
+                case CMD_DEL: {
                     d->m_packedArgs.truncate(0);
                     QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
                     stream << d->m_redirectionURL;
